import logging
from sys import stderr

from tornado.ioloop import IOLoop

from tfw.builtins import PipeIOEventHandler
from tfw.config import TFWENV
from tfw.logging import Log, Logger, LogFormatter, VerboseLogFormatter

from pipe_io_auxlib import (
        SignMessagePipeIOEventHandler, VerifyMessagePipeIOEventHandler,
        BotPipeIOEventHandler,
        DeployPipeIOEventHandler, IdePipeIOEventHandler,
        FSMPipeIOEventHandler
)
from signal_handling import setup_signal_handlers

LOG = logging.getLogger(__name__)


def main():
    # pylint: disable=unused-variable
    Logger([
        Log(stderr, LogFormatter(20)),
        Log(TFWENV.LOGFILE, VerboseLogFormatter())
    ]).start()

    json_pipe = PipeIOEventHandler(
        '',
        '/tmp/tfw_json_send',
        '/tmp/tfw_json_recv'
    )

    sign_pipe = SignMessagePipeIOEventHandler(
        '/tmp/tfw_sign_send',
        '/tmp/tfw_sign_recv',
        forwarding=True
    )

    verify_pipe = VerifyMessagePipeIOEventHandler(
        '/tmp/tfw_verify_send',
        '/tmp/tfw_verify_recv'
    )

    bot_pipe = BotPipeIOEventHandler(
        '/tmp/tfw_bot_send',
        '/tmp/tfw_bot_recv',
        permissions=0o666
    )

    deploy_pipe = DeployPipeIOEventHandler(
        '/tmp/tfw_deploy_send',
        '/tmp/tfw_deploy_recv',
        'webservice'
    )

    ide_pipe = IdePipeIOEventHandler(
        '/tmp/tfw_ide_send',
        '/tmp/tfw_ide_recv',
        'user_ops.py',
        selected=True
    )

    fsm_pipe = FSMPipeIOEventHandler(
        '/tmp/tfw_fsm_send',
        '/tmp/tfw_fsm_recv'
    )

    setup_signal_handlers()
    IOLoop.instance().start()


if __name__ == '__main__':
    main()
